/*
    Quick Log Entry (qle)
    Copyright (C) 2025  Thomas Cort

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <string.h>

#include "adif.h"
#include "parse.h"

void qle_adif_data_specifier(FILE *output, char *field, char *value) {
	fprintf(output, "<%s:%ld>%s", field, strlen(value), value);
}

void qle_adif_header(FILE *output) {

	fprintf(output, "# Converted from Quick Log Entry format %s to ADIF format %s by %s\n", QLE_VERSION, ADIF_VERSION, PACKAGE_STRING);
	qle_adif_data_specifier(output, "adif_version", ADIF_VERSION);
	fprintf(output, "\n");
	qle_adif_data_specifier(output, "programid", PACKAGE_NAME);
	fprintf(output, "\n");
	qle_adif_data_specifier(output, "programversion", PACKAGE_VERSION);
	fprintf(output, "\n");
	fprintf(output, "<eoh>\n\n");
}

void qle_adif_record(FILE *output, int compact) {

	int i;

	char *adif_fs;
	char *adif_rs;

	adif_fs = compact == 1 ? QLE_ADIF_FS_COMPACT : QLE_ADIF_FS_DEFAULT;
	adif_rs = compact == 1 ? QLE_ADIF_RS_COMPACT : QLE_ADIF_RS_DEFAULT;

	if (qle_fields[QLE_NFIELDS - 1].value[0] == '\0') {
		return; /* no call sign nothing to do */
	}

	for (i = 0; i < QLE_NFIELDS; i++) {
		if (qle_fields[i].value[0] != '\0') {
			qle_adif_data_specifier(output, qle_fields[i].adif_field, qle_fields[i].value);
			fprintf(output, "%s", adif_fs);
		}
	}
	fprintf(output, "<eor>%s", adif_rs);

	for (i = 0; i < QLE_NFIELDS; i++) {
		if (qle_fields[i].preserve == 0) {
			memset(qle_fields[i].value, '\0', QLE_TOKEN_SIZE);
		}
	}
	qle_rst_defaults();
}

