/*
    Quick Log Entry (qle)
    Copyright (C) 2025  Thomas Cort

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "args.h"
#include "parse.h"
#include "str.h"

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[]) {

	int ch;
	int compact;
	int rc;
	char *input_file;
	char *output_file;
	static char prelude[QLE_TOKEN_SIZE];
	FILE *input;
	FILE *output;
	FILE *error;
	char *mypota;
	char *mysota;
	char *mywwff;

	static struct option longopts[] = {
		{ "compact", no_argument, NULL, 'c' },
		{ "help", no_argument, NULL, 'h' },
		{ "pota", required_argument, NULL, 'p' },
		{ "sota", required_argument, NULL, 's' },
		{ "version", no_argument, NULL, 'V' },
		{ "wwff", required_argument, NULL, 'w' }
	};

	input_file = "-";
	output_file = "-";
	input = stdin;
	output = stdout;
	error = stderr;
	compact = 0;
	memset(prelude, '\0', QLE_TOKEN_SIZE);

	mypota = "MYPOTA@";
	mysota = "MYSOTA@";
	mywwff = "MYWWFF@";

	while ((ch = getopt_long(argc, argv, "chp:s:Vw:", longopts, NULL)) != -1) {
		switch (ch) {
			case 'c':
				compact = 1;
				break;
			case 'h':
				qle_help();
				return EXIT_SUCCESS;
			case 'p':
				qle_strnlcpy(prelude, mypota, QLE_TOKEN_SIZE, 0, strlen(mypota));
				qle_strnlcpy(prelude + strlen(mypota), optarg, QLE_TOKEN_SIZE - strlen(mypota), 0, strlen(optarg));
				break;
			case 's':
				qle_strnlcpy(prelude, mysota, QLE_TOKEN_SIZE, 0, strlen(mysota));
				qle_strnlcpy(prelude + strlen(mysota), optarg, QLE_TOKEN_SIZE - strlen(mysota), 0, strlen(optarg));
				break;
			case 'V':
				qle_version();
				return EXIT_SUCCESS;
			case 'w':
				qle_strnlcpy(prelude, mywwff, QLE_TOKEN_SIZE, 0, strlen(mywwff));
				qle_strnlcpy(prelude + strlen(mywwff), optarg, QLE_TOKEN_SIZE - strlen(mywwff), 0, strlen(optarg));
				break;
			default:
				qle_usage();
				return EXIT_FAILURE;
		}
	}

	argc -= optind;
	argv += optind;

	if (argc == 1) {
		input_file = argv[0];
	} else if (argc == 2) {
		input_file = argv[0];
		output_file = argv[1];
	}

	if (strcmp(input_file, "-") != 0) {
		input = fopen(input_file, "r");
		if (input == NULL) {
			fprintf(stderr, "Cannot Open Input Stream\n");
			return EXIT_FAILURE;
		}
	}
	if (strcmp(output_file, "-") != 0) {
		output = fopen(output_file, "w");
		if (output == NULL) {
			fprintf(stderr, "Cannot Open Output Stream\n");
			return EXIT_FAILURE;
		}
	}

	rc = qle_process(input, output, error, compact, prelude);

	fclose(input);
	fclose(output);
	fclose(error);

	return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}
