/*
    Quick Log Entry (qle)
    Copyright (C) 2025  Thomas Cort

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>

#include "band.h"
#include "str.h"
#include "transform.h"

void qle_transform_group(char *dst, char *src, int dstsize) {
	qle_strnlcpy(dst, src, dstsize, 1, strlen(src) - 1);
}

void qle_transform_no_last(char *dst, char *src, int dstsize) {
	qle_strnlcpy(dst, src, dstsize, 0, strlen(src) - 1);
}

void qle_transform_all(char *dst, char *src, int dstsize) {
	qle_strnlcpy(dst, src, dstsize, 0, strlen(src));
}

void qle_transform_freq(char *dst, char *src, int dstsize) {

	char *band;

	qle_strnlcpy(dst, src, dstsize, 0, strlen(src));

	band = qle_freq_to_band(src);
	if (band != NULL) {
		qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_BAND].value, band, dstsize, 0, strlen(band));
	}
}

void qle_transform_no_first(char *dst, char *src, int dstsize) {
	qle_strnlcpy(dst, src, dstsize, 1, strlen(src));
}

void qle_transform_sig(char *dst, char *src, int dstsize) {
	/* copy after 'POTA@' to POTA_REF */
	qle_strnlcpy(dst, src, dstsize, 5, strlen(src));
	/* copy 'POTA' to SIG */
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_SIG].value, src, dstsize, 0, 4);
	/* copy after 'POTA@' to SIG_INFO */
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_SIG_INFO].value, src, dstsize, 5, strlen(src));
}

void qle_transform_mysig(char *dst, char *src, int dstsize) {
	/* copy after 'MYPOTA@' to POTA_REF */
	qle_strnlcpy(dst, src, dstsize, 7, strlen(src));
	/* copy 'POTA' to SIG */
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_MY_SIG].value, src, dstsize, 2, 6);
	/* copy after 'MYPOTA@' to SIG_INFO */
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_MY_SIG_INFO].value, src, dstsize, 7, strlen(src));
}

void qle_transform_rst(char *dst, char *src, int dstsize) {
	(void) dst;
	/* copy rst_rcvd to rst_sent */
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_RST_SENT].value, qle_fields[QLE_FIELD_INDEX_RST_RCVD].value, dstsize, 0, strlen(qle_fields[QLE_FIELD_INDEX_RST_RCVD].value));
	/* write to rst_rcvd */
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_RST_RCVD].value, src, dstsize, 0, strlen(src));
}

void qle_transform_mode(char *dst, char *src, int dstsize) {
	qle_strnlcpy(dst, src, dstsize, 0, strlen(src));
	qle_rst_defaults();
}

